@@ -3,37 +3,40 @@
 #
 #	Computes the GNZ contrast of delta-f for any function f
 #
-#	$Revision: 1.5 $	$Date: 2013/04/25 06:37:43 $
+#	$Revision: 1.9 $	$Date: 2015/07/11 08:19:26 $
 #
 ################################################################################
 #
 
 psst <- function(object, fun, r=NULL, breaks=NULL, ...,
+                 model=NULL,
                  trend=~1, interaction=Poisson(),
                  rbord=reach(interaction),
                  truecoef=NULL, hi.res=NULL,
                  funargs=list(correction="best"),
                  verbose=TRUE) {
-  if(inherits(object, "ppm")) 
+  if(is.ppm(object)) {
     fit <- object
-  else if(inherits(object, "ppp"))
-    fit <- ppm(quadscheme(object, ...),
-               trend=trend, interaction=interaction, rbord=rbord)
-  else if(inherits(object, "quad")) 
-    fit <- ppm(object, 
-               trend=trend, interaction=interaction, rbord=rbord)
-  else 
+  } else if(is.ppp(object) || is.quad(object)) {
+    if(is.ppp(object)) object <- quadscheme(object, ...)
+    if(!is.null(model)) {
+      fit <- update(model, Q=object, forcefit=TRUE)
+    } else {
+      fit <- ppm(object, trend=trend, interaction=interaction, rbord=rbord,
+                 forcefit=TRUE)
+    }
+  } else 
     stop("object should be a fitted point process model or a point pattern")
 
-  rfixed <- !is.null(r) || !is.null(breaks)
+#  rfixed <- !is.null(r) || !is.null(breaks)
   
   # Extract data and quadrature points
   Q <- quad.ppm(fit, drop=FALSE)
   X <- data.ppm(fit)
   U <- union.quad(Q)
   Z <- is.data(Q) # indicator data/dummy
-  E <- equalsfun.quad(Q)
-  WQ <- w.quad(Q)  # quadrature weights
+#  E <- equalsfun.quad(Q)
+#  WQ <- w.quad(Q)  # quadrature weights
 
   # integrals will be restricted to quadrature points
   # that were actually used in the fit
@@ -45,23 +48,23 @@ psst <- function(object, fun, r=NULL, breaks=NULL, ...,
   } else USED <- rep.int(TRUE, U$n)
   
   # basic statistics
-  Win <- X$window
-  npoints <- X$n
-  area <- area.owin(Win)
-  lambda <- npoints/area
+  Win <- Window(X)
+  npts <- npoints(X)
+  areaW <- area(Win)
+  lambda <- npts/areaW
 
   # adjustments to account for restricted domain of pseudolikelihood
-  if(any(!USED)) {
-    XUSED <- USED[Z]
-    npoints.used <- sum(Z & USED)
-    area.used <- sum(WQ[USED])
-    lambda.used <- npoints.used/area.used
-  } else {
-    XUSED <- rep.int(TRUE, npoints)
-    npoints.used <- npoints
-    area.used <- area
-    lambda.used <- lambda
-  }
+#  if(any(!USED) && spatstat.options("eroded.intensity")) {
+#    XUSED <- USED[Z]
+#    npts.used <- sum(Z & USED)
+#    area.used <- sum(WQ[USED])
+#    lambda.used <- npts.used/area.used
+#  } else {
+#    XUSED <- rep.int(TRUE, npts)
+#    npts.used <- npts
+#    area.used <- areaW
+#    lambda.used <- lambda
+#  }
   
   #  determine breakpoints for r values
   rmaxdefault <- rmax.rule("G", Win, lambda)
@@ -101,7 +104,9 @@ psst <- function(object, fun, r=NULL, breaks=NULL, ...,
   uX <- superimpose(U[1], X, W=Win, check=FALSE)
   Ux <- U$x
   Uy <- U$y
-  # 
+  #
+  if(verbose) pstate <- list()
+  #
   for(j in seq(nused)) {
     i <- iused[j]
     wi <- wc[i]
@@ -127,7 +132,7 @@ psst <- function(object, fun, r=NULL, breaks=NULL, ...,
       gc()
       cat("collected]")
     }
-    if(verbose) progressreport(j, nused)
+    if(verbose) pstate <- progressreport(j, nused, state=pstate)
   }
 
   sdv <- sqrt(integ2)
